// on importe la bibliotheque opencv
import hypermedia.video.*;
import java.awt.*;

// on créée un objet opencv
OpenCV opencv;

// on créée des variables pour faciliter la mosaique des images de captation pour plus tard
int w = 320;
int h = 240;
// on créée une variable pour initiliser la valeur de contraste de l'image en bas à droite
int threshold = 80;

// on crée une variable booléenne
boolean find=true;

// on créée une variable police de caractère pour afficher les valeurs (tailles, surfaces, centre de gravité...) des blobs
PFont font;

void setup() {
  // on détermine la taille de notre sketch à partir de celles des différents écrans
  size( w*2+30, h*2+30 );

  // on initialise l'objet OpenCV
  opencv = new OpenCV( this );
  // on créée la fenêtre de capture
  opencv.capture(w, h);

  // je charge la police de caracteres associees à la variable font
  font = loadFont( "AndaleMono.vlw" );
  // je détermine comme la variable font comme police de caractères par défaut
  textFont( font );

  // j'imprime dans la console de debuggage les instructions suivantes :
  println( "Drag mouse inside sketch window to change threshold" );
  println( "Press space bar to record background image" );
}



void draw() {
  //je détermine ma couleur de fond (noir)
  background(0);
  //je lance la lecture de ce que capte la camera
  opencv.read();
  // je desactive la fonction mirroir d'affichage
  //opencv.flip( OpenCV.FLIP_HORIZONTAL );

  // ma fenêtre de capture en haut à gauche (poisition X : 10, position Y : 10)
  image( opencv.image(), 10, 10 );	            // RGB image
  // ma fenêtre de capture en haut à droite 
  image( opencv.image(OpenCV.GRAY), 20+w, 10 );   // GRAY image
  // ma fenêtre snapshot. ne s'affiche que lorsque que je fais un instantané en appuyant sur la barre espace du clavier
  image( opencv.image(OpenCV.MEMORY), 10, 20+h ); // image in memory

  //
  opencv.absDiff();
  //j'active la fonction de contraste et lui attribue la valeur de la variable 'treshold'
  opencv.threshold(threshold);
  // Pour comparer notre instantané avec la fenêtre 
  image( opencv.image(OpenCV.GRAY), 20+w, 20+h ); // absolute difference image


  // on lance la detection de blobs sous forme de liste.
  // les paramètres de opencv.blobs()
  Blob[] blobs = opencv.blobs( 100, w*h/3, 20, true );

  // on desactive le remplissage de toutes les formes qui vont suivre dans le programme
  noFill();

  pushMatrix();
  //on déplace notre 'calque' de contour de blob pour 
  //le faire se superposer avec notre fenetre de captation 
  //de camera en bas à droite
  translate(20+w, 20+h);

  //on créée un boucle d'itération pour détecter de manière 
  //dynamique TOUTES les surafces les plus claires 
  //de la captation de la webcam
  for ( int i=0; i<blobs.length; i++ ) {

    //on initialise la fonction de la bibliothèque opencv
    // qui s'appelle Rectangle et qui permet de
    //dessiner les rectangle autour de nos blobs
    Rectangle bounding_rect	= blobs[i].rectangle;
    //on créée une variable nommée 'area'
   // à laquelle on donne la valeur de la surface de nos blobs grâce 
   //à la fonction '.area'
   //de la bibliothèque opencv
    float area = blobs[i].area;
    //on créée un objet pour récupérér la valeur de la surface des blobs
    float circumference = blobs[i].length;
    // à chaque blobs, on active la fonction .centroid
    // de la bibliothèque OpenCV
    //ça va nous permettre de récupérer les positions X et Y
    // des centres de gravité de nos blobs
    Point centroid = blobs[i].centroid;
    Point[] points = blobs[i].points;

    // Dessins des rectangles autour de nos blobs
    // on désactive la fonction de remplissage
    noFill();
    //on dessine un contour en demandant à la bibliothèque opencv
    // par la fonction .isHole : on s'assure que c'est ou non un blob dans un autre blob
    stroke( blobs[i].isHole ? 128 : 64 );
    //on dessine les rectangles en recuperant les coordonnées par
    //les variables bounding_rect.x et bounding_rect.y
   // et la hauteur et la largeur par les variables bounding_rect.width et bounding_rect.height
    rect( bounding_rect.x, bounding_rect.y, bounding_rect.width, bounding_rect.height );


      // CENTROID / CENTRES DE GRAVITÉ
      //on créée un contour de couleur rouge
    stroke(0, 0, 255);
    //on dessine la ligne vhorizontale de la croix
    // qui indique le centre de gravité du blob
    // cendroid.x : position X Y du centre de gravité du blob
    // centroid.y : position Y du centre de gravité du blob
    line( centroid.x-5, centroid.y, centroid.x+5, centroid.y );
    //on dessine la ligne vhorizontale de la croix
    // qui indique le centre de gravité du blob
    // cendroid.x : position X Y du centre de gravité du blob
    // centroid.y : position Y du centre de gravité du blob
    line( centroid.x, centroid.y-5, centroid.x, centroid.y+5 );
    // on desactive les contours
    noStroke();
    //on active une nouvelle couleur de remplissage
    fill(0, 0, 255);
    // on écrit la valeur de la surface (variable area) du blob, 
    // à coté de notre croix
    //grâce aux positions du centre de gravité
    // position X : centroid.x
    // position Y : centroid.y
    text( area, centroid.x+5, centroid.y+5 );

    // couleur de remplissage des dessins des formes de blobs
    fill(255, 0, 255, 64);
    // couleur de contours des dessins des formes de blobs
    stroke(255, 0, 255);
    // début des dessins de formes de blobs
    if ( points.length>0 ) {
      beginShape();
      for ( int j=0; j<points.length; j++ ) {
        vertex( points[j].x, points[j].y );
      }
      endShape(CLOSE);
      // fin des dessins de formes de blobs
    }

// on desactive le dessin des contours
    noStroke();
    //on choisit une nouvelle couleur de remplissage
    fill(255, 0, 255);
    // on ecrit les textes d'information en récupérant les valeurs
    // circumference : taille du périmètre du contour
    // en les plaçant à coté du curseur
    // grace au coordonnées des points de gravité (centroid.x, entroid.y)
    text( circumference, centroid.x+5, centroid.y+15 );
  }
  popMatrix();
}

void keyPressed() {
  // quand on appuie sur la barre espace du clavier, on lance la fonction opencv.remember de la bibliothèque Opencv
  if ( key==' ' ) opencv.remember();
}

void mouseDragged() {
  // quand on déplace la souris, on change la valeur du contraste 
  threshold = int( map(mouseX, 0, width, 0, 255) );
}

public void stop() {
  // une fonction qui arrête tous les processus en cours lors de l'extinction de notre sketch
  opencv.stop();
  super.stop();
}

